/*  Copyright (C) 1996 Luis Falcon (lfalcon@csun.edu)
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include <curses.h>
#include <math.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "./defaults.h"

#define CMD_LINE  fprintf (stderr,"usage : netload device [-t secs] \n")

WINDOW *panel;
WINDOW *meter;
WINDOW *lever;
WINDOW *devcred;
WINDOW *INFO;

short int INIT = 0;
int old_rec=0;
int old_trans=0;
int max_sofar=0;

int check_arguments (int c, char *v[]);
void show_load (char *line, int speed, int dormant);
void reset (int);
char *current_time ();


void main (int argc, char *argv[])
{

FILE *source;
char *line;
char content [10000];
double filesize;
int dormant;
int speed;
int argument_length;
char mask [6];
check_arguments (argc, argv);

signal (SIGINT,reset);

if (argc > 2)
  dormant = atoi (argv [3]);
else
  dormant = 1;

if ((strncmp (argv[1],"ppp",3) == 0) | (strncmp (argv[1],"sl",2) == 0))
   speed = SERIAL;
else
   speed = ETHER;

line = malloc (100*sizeof (char));

initscr ();
nonl();
intrflush(stdscr, FALSE);
keypad(stdscr, TRUE);
leaveok (stdscr, TRUE); // hide cursor
noecho ();
start_color ();

init_pair (COLOR_CYAN, COLOR_CYAN, COLOR_BLACK);
init_pair (COLOR_MAGENTA, COLOR_MAGENTA, COLOR_BLACK );
init_pair (COLOR_RED, COLOR_RED, COLOR_BLACK );
init_pair (COLOR_GREEN, COLOR_GREEN, COLOR_BLACK);
init_pair (COLOR_BLUE, COLOR_BLUE, COLOR_BLACK);
init_pair (COLOR_YELLOW, COLOR_YELLOW, COLOR_BLACK);

strcpy (mask,argv[1]);
argument_length = strlen (argv[1]);
mask [argument_length]=':';

INFO = newwin (6,80,1,0);
panel = newwin (10,60,7,10);
meter = newwin (3,50,17,15);
lever = newwin (1,48,18,16);
devcred = newwin (3,71,21,3);

while (1)
  {
    if ((source=fopen ("/proc/net/dev","r")) == NULL)
      {
	fprintf (stderr, "Couldn't open /proc/net/dev.\n");
	endwin ();
	exit (1);
      }
    filesize = lseek ((int)source,0,SEEK_END);
    fread (content,filesize,1,source);
    if ((line = strstr (content,mask)) == NULL)
      {
	fprintf (stderr,"Device \"%s\" not found.\n",argv[1]);
	endwin ();
	exit (1);
      }
  show_load (line,speed,dormant);
  fclose (source);
  sleep (dormant);
  }

}

int check_arguments (int c, char *v[])
{
    if (c < 2 || c > 4 || c == 3)
      {
	CMD_LINE;
	exit (1);
      }
    if (c==4 && strcmp (v[2],"-t") != 0)
      {
	CMD_LINE;
	exit (1);
      }
    if (c==4 && strcmp (v[2],"-t") == 0 && atoi (v[3]) <= 0)
      {
	fprintf (stderr, "Time must be >= 1 sec.\n");
	exit (1);
      }
    return (0);
}

void show_load (char *line, int speed, int dormant)
{

char device [5];
short int marks;
int packets,rpackets;
int    errors,drop,fifo,frame;
int    rerrors,rdrop,rfifo,rframe,carrier;
int rec_load, trans_load;
int total_load;
int load_level;

    sscanf (line,"%[^:]: %d %d %d %d %d %d %d %d %d %d %d\n",device,&rpackets,&rerrors,&rdrop,&rfifo,&rframe,&packets,&errors,&drop,&fifo,&frame,&carrier);

    if (INIT == 0)
      {
	box (INFO,0,0);
	wattrset (INFO,COLOR_PAIR (COLOR_MAGENTA) | A_BOLD);
	mvwprintw (INFO,1,3,"Netload ver. %s", VERSION);
	wattrset (INFO,COLOR_PAIR (COLOR_CYAN));
	mvwprintw (INFO,1,25,"PACKETS     ERRORS       DROP       FIFO      FRAME");
	wattrset (INFO,COLOR_PAIR (COLOR_BLUE));
	wmove (INFO,2,21);
	whline (INFO,0,56);
	wattrset (INFO,COLOR_PAIR (COLOR_RED) | A_BOLD);
	mvwprintw (INFO,3,2,"TOTAL RECEIVED:");
	mvwprintw (INFO,4,2,"TOTAL TRANSMITTED:");
	wattron (INFO,COLOR_PAIR (COLOR_GREEN));

	wattrset (meter,COLOR_PAIR (COLOR_RED));
	box (meter,0,0);
	wnoutrefresh (meter);

	wattrset (panel,COLOR_PAIR (COLOR_YELLOW));
	wmove (panel,2,1);
	box (panel,0,0);
	wattron (panel, A_BOLD);
	mvwprintw (panel,3,9,"Relative load in %.10d sec(s) steps:",dormant);
	wattron (panel,COLOR_PAIR (COLOR_RED));
	mvwprintw (panel,1,11,"Start time: %s", current_time ());
	wattron (panel,COLOR_PAIR (COLOR_GREEN));
	wmove (panel,4,7);
	whline (panel,0,45);
	wattrset (panel,A_BOLD);
	mvwprintw (panel,5,4,"MAX. PEAK-LOAD:\t   0 (%s)", current_time ());

	wattrset (devcred,COLOR_PAIR (COLOR_CYAN) | A_BOLD);
	mvwprintw (devcred,0,30,"Device %s",device);
	wattron (devcred,COLOR_PAIR (COLOR_BLUE));
	mvwprintw (devcred,2,0,"By Luis Falcon (lfalcon@csun.edu), changes by Jan Rafaj (rafaj@vabo.cz)");
	wnoutrefresh (devcred);

	old_rec = rpackets;
	old_trans = packets;
    }

    mvwprintw (INFO,3,22,"%10d %10d %10d %10d %10d",rpackets,rerrors,rdrop,rfifo,rframe);
    mvwprintw (INFO,4,22,"%10d %10d %10d %10d %10d",packets,errors,drop,fifo,frame);

    rec_load = rpackets - old_rec;
    old_rec = rpackets;

    trans_load = packets - old_trans;
    old_trans = packets;

    total_load = (rec_load + trans_load)/dormant;

    if (total_load >= max_sofar)
      {
        max_sofar = total_load;
	mvwprintw (panel,5,22,"%6d (%s)",max_sofar,current_time ());
      }
    mvwprintw (panel,2,10,"Current time: %s",current_time ());
    mvwprintw (panel,6,4,"AVERAGE LOAD: %10d packets / 1 second",total_load);
    wattron (panel,COLOR_PAIR (COLOR_CYAN));
    mvwprintw (panel,7,4,"RECEIVED:     %10d packets / %d sec(s)",rec_load,dormant);
    mvwprintw (panel,8,4,"TRANSMITTED:  %10d packets / %d sec(s)",trans_load,dormant);
    wattrset (panel,A_BOLD);

    werase (lever);

    load_level = total_load/speed;

    if (load_level > 48)
	load_level = 48;

    if (total_load > 0)
      {
	if (load_level <=16)
		wattrset (lever, A_BOLD);

	if (load_level >16 && load_level < 32)
		wattrset (lever,COLOR_PAIR (COLOR_GREEN));

	if (load_level > 32)
		wattrset (lever,COLOR_PAIR (COLOR_MAGENTA));

      	for (marks=0;marks < load_level;marks++)
			waddch (lever,ACS_BLOCK);
      }

    wnoutrefresh (INFO);
    wnoutrefresh (panel);
    wnoutrefresh (lever);

    doupdate();

    INIT =1;
}

char *current_time ()
{
    time_t reloj;
    char *time_holder;
    time (&reloj);
    time_holder = ctime (&reloj);
    time_holder [24] =0;
    return (time_holder);
}

void reset (int signal)
{
    endwin ();
    printf ("Later...\n");
    exit (0);
}

